@@ -19,18 +19,27 @@ try:
 except ImportError:
     pytz = None
 
-__version__ = '0.0.3'
+__version__ = '0.0.4'
 
-# Todo: Build out the persistence support, so it can run with various backends (RDBMS, Redis, Celery, etc.).
-# Todo: Move calculation of value based on brownian motion from Market object, so that its evaluation method simply looks up value of random variable for that market at that present time.
 # Todo: Develop PriceSimulation object from just a simple "local" in memory, to be an network service client object.
-# Todo: Check whether it's better to regress on uncorrelated brownian motions, rather than correlated ones.
+# Todo: Make price process by configurable on command line, with option for stating which Python class to use?
+# Todo: Check whether it's okay to regress on correlated brownian motions, rather than uncorrelated ones (if so, no need to keep the uncorrelated once correlated are generated).
+# Todo: Separate multiprocessing from ExpressionStack, self-evaluation of ExpressionStack can just be single threaded.
+# Todo: Develop the multiprocessing code into a stack runner object, which can be replaced with a network-based runner?
+# Todo: Build out the persistence support, so it can run with various backends (RDBMS, Redis, Celery, etc.).
+# Todo: Develop a GUI that shows the graph being evaluated, allowing results to be examined, allows models to be developed.
+# Todo: Make stats available on number of call requirements, number of leaves in dependency graph, depth of graph?
+# Todo: Prediction of cost of evaluating an expression, cost of network data requests, could calibrate by running sample stubbed expressions (perhaps complicated for LongstaffSchwartz cos the LeastSqaures routine is run different numbers of times).
+# Todo: Unsubscribe from results, and delete intermediate result when there are zero subscriptions (or subscribe to consumers)?
+# Todo: Move to an event sourced model for dependency graph changes (CreatedResult, CreatedCallDependency, CreatedCallRequirement, etc.) and use an RDBMS (even sqlite in memory) to manage a single table with everything in.
 # Todo: Develop multi-factor model (e.g. Schwartz-Smith)?
+# Todo: Develop natural language "skin" for Quant DSL expressions (something like how Gherkin syntax maps to functions?)?
+
+# Todo: Introduce support for "import" statements, so that a library of Quant DSL can be developed (so e.g. Option can be shared). Need to have a path system? And a packaging and distribution system? Perhaps just make them Python modules? Perhaps just start with everything in one QUANTDSL_SOURCE_PATH?
+
 # Todo: Develop closures, function defs within function defs may help to reduce call argument complexity.
-# Todo: Make stats available on number of call requirements, number of leaves in dependency graph. And depth of graph? Predicting cost of each node would be possible (but complicated for LongstaffSchwartz cos the LeastSqaures routine is run different numbers of times (spot the formula).
 # Todo: Optimization for parallel execution, so if there are four cores, then it might make sense only to stub four large branches?
 # Todo: Optimize network traffic by creating a single message containing all data required to evaluate a stubbed expression.
-# Todo: Move to an event sourced model for dependency graph changes (CreatedResult, CreatedCallDependency, CreatedCallRequirement, etc.) and use an RDBMS (even sqlite in memory) to manage a single table with everything in.
 # Todo: Clean up the str, repr, pprint stuff?
 # Todo: Make this works with Python 3.
 # Todo: Use function arg annotation to declare types of DSL function args (will only work with Python 3).
@@ -81,7 +90,7 @@ def compile(dslSource, filename='<unknown>', isParallel=None, dslClasses=None, c
 
 
 def eval(dslSource, filename='<unknown>', isParallel=None, dslClasses=None, compileKwds=None, evaluationKwds=None,
-         priceSimulatorClass=None, isVerbose=False, **extraEvaluationKwds):
+         priceProcessName='quantdsl:BlackScholesPriceProcess', isVerbose=False, **extraEvaluationKwds):
     """
     Returns the result of evaluating a compiled module (an expression, or a user defined function).
 
@@ -152,6 +161,7 @@ def eval(dslSource, filename='<unknown>', isParallel=None, dslClasses=None, comp
             if isVerbose:
                 print "Finding all market names and fixing dates..."
                 print
+
             # Find all unique market names.
             marketNames = set()
             for dslMarket in dslExpr.findInstances(dslType=Market):
@@ -174,11 +184,43 @@ def eval(dslSource, filename='<unknown>', isParallel=None, dslClasses=None, comp
                 )
                 print
 
-            if not priceSimulatorClass:
-                priceSimulatorClass = PriceSimulator
-            priceSimulator = priceSimulatorClass()
-            brownianMotions = priceSimulator.getBrownianMotions(marketNames, fixingDates, observationTime, pathCount, marketCalibration)
-            evaluationKwds['brownianMotions'] = brownianMotions
+            # Load the price process object.
+            if not ':' in priceProcessName:
+                raise QuantDslError("Price process name doesn't have ':' separating module and class names: %s" % priceProcessName)
+            priceProcessModuleName, priceProcessClassName = priceProcessName.split(':')
+            try:
+                priceProcessModule = __import__(priceProcessModuleName, '', '', '*')
+            except Exception, e:
+                raise QuantDslError("Can't import price process module '%s': %s" % (priceProcessModuleName, e))
+            try:
+                priceProcessClass = getattr(priceProcessModule, priceProcessClassName)
+            except Exception, e:
+                raise QuantDslError("Can't find price process class '%s' in module '%s': %s" % (priceProcessClassName, priceProcessModuleName, e))
+
+            assert issubclass(priceProcessClass, PriceProcess)
+
+            priceProcess = priceProcessClass()
+
+            # allBrownianMotions = priceProcess.getBrownianMotions(marketNames, fixingDates, observationTime, pathCount, marketCalibration)
+            # evaluationKwds['brownianMotions'] = allBrownianMotions
+            #
+            # # Also compute market prices, so the Market object doesn't do this.
+            # allMarketPrices = {}
+            # for (marketName, brownianMotions) in allBrownianMotions.items():
+            #     actualHistoricalVolatility = marketCalibration['%s-ACTUAL-HISTORICAL-VOLATILITY' % marketName.upper()]
+            #     lastPrice = marketCalibration['%s-LAST-PRICE' % marketName.upper()]
+            #     marketPrices = {}
+            #
+            #     for (fixingDate, brownianRv) in brownianMotions.items():
+            #         sigma = actualHistoricalVolatility / 100
+            #         T = getDurationYears(observationTime, fixingDate)
+            #         marketRv = lastPrice * scipy.exp(sigma * brownianRv - 0.5 * sigma * sigma * T)
+            #         marketPrices[fixingDate] = marketRv
+            #
+            #     allMarketPrices[marketName] = marketPrices
+
+        allMarketPrices = priceProcess.simulateFuturePrices(marketNames, fixingDates, observationTime, pathCount, marketCalibration)
+        evaluationKwds['allMarketPrices'] = allMarketPrices
 
 
     if isinstance(dslExpr, ExpressionStack):
@@ -190,23 +232,23 @@ def eval(dslSource, filename='<unknown>', isParallel=None, dslClasses=None, comp
 
             print "Evaluating %d partial expressions, please wait..." % lenDslExpr
             print
-        # Start progress display thread.
-        def showProgress():
-            progress = 0
-            while progress < 100:
-                time.sleep(0.2)
-                # Avoid race condition.
-                if hasattr(dslExpr, 'runner') and hasattr(dslExpr.runner, 'resultsDict'):
-                    progress = 100.0 * len(dslExpr.runner.resultsDict) / lenDslExpr
-                    sys.stdout.write("\rProgress: %01.2f%%" % progress)
-                    sys.stdout.flush()
-                else:
-                    time.sleep(0.5)
-            sys.stdout.write("\r")
-            sys.stdout.flush()
+            # Start progress display thread.
+            def showProgress():
+                progress = 0
+                while progress < 100:
+                    time.sleep(0.2)
+                    # Avoid race condition.
+                    if hasattr(dslExpr, 'runner') and hasattr(dslExpr.runner, 'resultsDict'):
+                        progress = 100.0 * len(dslExpr.runner.resultsDict) / lenDslExpr
+                        sys.stdout.write("\rProgress: %01.2f%%" % progress)
+                        sys.stdout.flush()
+                    else:
+                        time.sleep(0.5)
+                sys.stdout.write("\r")
+                sys.stdout.flush()
 
-        thread = threading.Thread(target=showProgress)
-        thread.start()
+            thread = threading.Thread(target=showProgress)
+            thread.start()
 
     # Evaluate the primitive DSL expression.
     startEval = datetime.datetime.now()
@@ -214,8 +256,9 @@ def eval(dslSource, filename='<unknown>', isParallel=None, dslClasses=None, comp
     durationEval = datetime.datetime.now() - startEval
 
     if isinstance(dslExpr, ExpressionStack):
-        # Join with progress display thread.
-        thread.join(timeout=3)
+        if isVerbose:
+            # Join with progress display thread.
+            thread.join(timeout=3)
 
     if isVerbose:
         print "Duration of evaluation: %s" % durationEval
@@ -301,7 +344,7 @@ class DslParser(object):
         """
         assert isinstance(node, ast.Module)
         body = [self.visitAstNode(n) for n in node.body]
-        return Module(body, node=node)
+        return self.dslClasses['Module'](body, node=node)
 
     def visitExpr(self, node):
         """
@@ -322,7 +365,7 @@ class DslParser(object):
         Returns a DSL Number object, with the number value.
         """
         assert isinstance(node, ast.Num)
-        return Number(node.n, node=node)
+        return self.dslClasses['Number'](node.n, node=node)
 
     def visitStr(self, node):
         """
@@ -331,7 +374,7 @@ class DslParser(object):
         Returns a DSL String object, with the string value.
         """
         assert isinstance(node, ast.Str)
-        return String(node.s, node=node)
+        return self.dslClasses['String'](node.s, node=node)
 
     def visitUnaryOp(self, node):
         """
@@ -342,7 +385,7 @@ class DslParser(object):
         assert isinstance(node, ast.UnaryOp)
         args = [self.visitAstNode(node.operand)]
         if isinstance(node.op, ast.USub):
-            dslUnaryOpClass = UnarySub
+            dslUnaryOpClass = self.dslClasses['UnarySub']
         else:
             raise QuantDslSyntaxError("Unsupported unary operator token: %s" % node.op)
         return dslUnaryOpClass(node=node, *args)
@@ -355,13 +398,13 @@ class DslParser(object):
         """
         assert isinstance(node, ast.BinOp)
         typeMap = {
-            ast.Add: Add,
-            ast.Sub: Sub,
-            ast.Mult: Mult,
-            ast.Div: Div,
-            ast.Pow: Pow,
-            ast.Mod: Mod,
-            ast.FloorDiv: FloorDiv,
+            ast.Add: self.dslClasses['Add'],
+            ast.Sub: self.dslClasses['Sub'],
+            ast.Mult: self.dslClasses['Mult'],
+            ast.Div: self.dslClasses['Div'],
+            ast.Pow: self.dslClasses['Pow'],
+            ast.Mod: self.dslClasses['Mod'],
+            ast.FloorDiv: self.dslClasses['FloorDiv'],
         }
         try:
             dslClass = typeMap[type(node.op)]
@@ -378,8 +421,8 @@ class DslParser(object):
         """
         assert isinstance(node, ast.BoolOp)
         typeMap = {
-            ast.And: And,
-            ast.Or: Or,
+            ast.And: self.dslClasses['And'],
+            ast.Or: self.dslClasses['Or'],
         }
         try:
             dslClass = typeMap[type(node.op)]
@@ -396,7 +439,7 @@ class DslParser(object):
 
         Returns a DSL Name object, along with the name's string.
         """
-        return Name(node.id, node=node)
+        return self.dslClasses['Name'](node.id, node=node)
 
     def visitCall(self, node):
         """
@@ -428,7 +471,7 @@ class DslParser(object):
             # Resolve as a FunctionCall, and expect
             # to resolve the name to a function def later.
             dslArgs = [Name(calledNodeName, node=calledNode), callArgExprs]
-            return FunctionCall(node=node, *dslArgs)
+            return self.dslClasses['FunctionCall'](node=node, *dslArgs)
         else:
             assert issubclass(dslClass, DslObject)
             return dslClass(node=node, *callArgExprs)
@@ -445,7 +488,7 @@ class DslParser(object):
         decoratorNames = [astName.id for astName in node.decorator_list]
         body = self.visitAstNode(node.body[0])
         dslArgs = [name, callArgDefs, body, decoratorNames]
-        functionDef = FunctionDef(node=node, *dslArgs)
+        functionDef = self.dslClasses['FunctionDef'](node=node, *dslArgs)
         return functionDef
 
     def visitIfExp(self, node):
@@ -459,7 +502,7 @@ class DslParser(object):
         body = self.visitAstNode(node.body)
         orelse = self.visitAstNode(node.orelse)
         args = [test, body, orelse]
-        return IfExp(node=node, *args)
+        return self.dslClasses['IfExp'](node=node, *args)
 
     def visitIf(self, node):
         """
@@ -475,7 +518,7 @@ class DslParser(object):
             node.orelse) == 1, "If statements with more than one orelse statement are not supported at the moment."
         orelse = self.visitAstNode(node.orelse[0])
         args = [test, body, orelse]
-        return If(node=node, *args)
+        return self.dslClasses['If'](node=node, *args)
 
     def visitCompare(self, node):
         """
@@ -488,7 +531,7 @@ class DslParser(object):
         opNames = [o.__class__.__name__ for o in node.ops]
         comparators = [self.visitAstNode(c) for c in node.comparators]
         args = [left, opNames, comparators]
-        return Compare(node=node, *args)
+        return self.dslClasses['Compare'](node=node, *args)
 
 
 class DslObject(object):
@@ -705,15 +748,18 @@ utc = pytz.utc if pytz else UTC()
 
 class Date(DslConstant):
 
-    requiredType = (String, datetime.datetime)
+    requiredType = (basestring, String, datetime.datetime)
 
     def __str__(self, indent=0):
         return "Date('%04d-%02d-%02d')" % (self.value.year, self.value.month, self.value.day)
 
     def parse(self, value):
         # Return a datetime.datetime.
-        if isinstance(value, String):
-            dateStr = value.evaluate()
+        if isinstance(value, (basestring, String)):
+            if isinstance(value, String):
+                dateStr = value.evaluate()
+            else:
+                dateStr = value
             try:
                 return dateutil.parser.parse(dateStr).replace(tzinfo=utc)
             except ValueError, inst:
@@ -946,22 +992,13 @@ class Max(BinOp):
 class Market(DslExpression):
     def validate(self, args):
         self.assertArgsLen(args, requiredLen=1)
-        self.assertArgsPosn(args, posn=0, requiredType=(String, Name))
+        self.assertArgsPosn(args, posn=0, requiredType=(basestring, String, Name))
 
     @property
     def name(self):
-        return self._args[0].evaluate()
+        return self._args[0].evaluate() if isinstance(self._args[0], String) else self._args[0]
 
     def evaluate(self, **kwds):
-        # Todo: Improve on having various ways of checking variables are defined.
-        try:
-            observationTime = kwds['observationTime']
-        except KeyError:
-            raise QuantDslSyntaxError(
-                "Can't evaluate Market '%s' without 'observationTime' in context variables." % self.name,
-                ", ".join(kwds.keys()),
-                node=self.node
-            )
         try:
             presentTime = kwds['presentTime']
         except KeyError:
@@ -971,67 +1008,114 @@ class Market(DslExpression):
                 node=self.node
             )
         try:
-            calibration = kwds['marketCalibration']
+            allMarketPrices = kwds['allMarketPrices']
         except KeyError:
             raise QuantDslError(
-                "Can't evaluate Market '%s' without 'calibration' in context variables" % self.name,
+                "Can't evaluate Market '%s' without 'allMarketPrices' in context variables" % self.name,
                 ", ".join(kwds.keys()),
                 node=self.node
             )
 
-        # Todo: Move this price construction into the PriceSimulator, and make Market values available by (name, date).
-
-        LAST_PRICE = '%s-LAST-PRICE' % self.name
         try:
-            lastPrice = calibration[LAST_PRICE]
+            marketPrices = allMarketPrices[self.name]
         except KeyError:
             raise QuantDslError(
-                "Can't evaluate Market '%s' without calibration setting '%s" % (self.name, LAST_PRICE),
-                ", ".join(calibration.keys()),
-                node=self.node
-            )
-        ACTUAL_HISTORICAL_VOLATILITY = '%s-ACTUAL-HISTORICAL-VOLATILITY' % self.name
-        try:
-            actualHistoricalVolatility = calibration[ACTUAL_HISTORICAL_VOLATILITY]
-        except KeyError:
-            raise QuantDslError(
-                "Can't evaluate Market '%s' without calibration setting '%s" % (self.name, actualHistoricalVolatility),
-                ", ".join(calibration.keys()),
+                "Can't evaluate Market '%s' without market name in 'allMarketPrices'" % self.name,
+                ", ".join(allMarketPrices.keys()),
                 node=self.node
             )
 
-        if presentTime == observationTime:
-            value = lastPrice
-        else:
-            try:
-                brownianMotions = kwds['brownianMotions']
-            except KeyError:
-                raise QuantDslSyntaxError(
-                    "Can't evaluate Market '%s' without 'brownianMotions' in context variables" % self.name,
-                    ", ".join(kwds.keys()),
-                    node=self.node
-                )
+        try:
+            marketPrice = marketPrices[presentTime]
+        except KeyError:
+            raise QuantDslError(
+                "Can't evaluate Market '%s' without present time '%s in market prices" % (self.name, presentTime),
+                ", ".join(marketPrices.keys()),
+                node=self.node
+            )
 
-            try:
-                marketRvs = brownianMotions[self.name]
-            except KeyError:
-                raise QuantDslSyntaxError(
-                    "Market '%s' not available in rvs" % self.name,
-                    ", ".join(brownianMotions.keys()),
-                    node=self.node
-                )
-
-            try:
-                marketRv = marketRvs[presentTime]
-            except KeyError:
-                raise Exception, "Present time %s not in market rvs: %s" % (presentTime, marketRvs.keys())
-
-            sigma = actualHistoricalVolatility / 100
-            import scipy
-            T = getDurationYears(observationTime, presentTime)
-            value = lastPrice * scipy.exp(sigma * marketRv - 0.5 * sigma * sigma * T)
-        return value
+        return marketPrice
 
+    # def _evaluate(self, **kwds):
+    #     # Todo: Improve on having various ways of checking variables are defined.
+    #     try:
+    #         observationTime = kwds['observationTime']
+    #     except KeyError:
+    #         raise QuantDslSyntaxError(
+    #             "Can't evaluate Market '%s' without 'observationTime' in context variables." % self.name,
+    #             ", ".join(kwds.keys()),
+    #             node=self.node
+    #         )
+    #     try:
+    #         presentTime = kwds['presentTime']
+    #     except KeyError:
+    #         raise QuantDslSyntaxError(
+    #             "Can't evaluate Market '%s' without 'presentTime' in context variables" % self.name,
+    #             ", ".join(kwds.keys()),
+    #             node=self.node
+    #         )
+    #     try:
+    #         calibration = kwds['marketCalibration']
+    #     except KeyError:
+    #         raise QuantDslError(
+    #             "Can't evaluate Market '%s' without 'calibration' in context variables" % self.name,
+    #             ", ".join(kwds.keys()),
+    #             node=self.node
+    #         )
+    #
+    #     # Todo: Move this price construction into the BlackScholesPriceProcess, and make Market values available by (name, date).
+    #
+    #     LAST_PRICE = '%s-LAST-PRICE' % self.name
+    #     try:
+    #         lastPrice = calibration[LAST_PRICE]
+    #     except KeyError:
+    #         raise QuantDslError(
+    #             "Can't evaluate Market '%s' without calibration setting '%s" % (self.name, LAST_PRICE),
+    #             ", ".join(calibration.keys()),
+    #             node=self.node
+    #         )
+    #     ACTUAL_HISTORICAL_VOLATILITY = '%s-ACTUAL-HISTORICAL-VOLATILITY' % self.name
+    #     try:
+    #         actualHistoricalVolatility = calibration[ACTUAL_HISTORICAL_VOLATILITY]
+    #     except KeyError:
+    #         raise QuantDslError(
+    #             "Can't evaluate Market '%s' without calibration setting '%s" % (self.name, actualHistoricalVolatility),
+    #             ", ".join(calibration.keys()),
+    #             node=self.node
+    #         )
+    #
+    #     if presentTime == observationTime:
+    #         value = lastPrice
+    #     else:
+    #         try:
+    #             brownianMotions = kwds['brownianMotions']
+    #         except KeyError:
+    #             raise QuantDslSyntaxError(
+    #                 "Can't evaluate Market '%s' without 'brownianMotions' in context variables" % self.name,
+    #                 ", ".join(kwds.keys()),
+    #                 node=self.node
+    #             )
+    #
+    #         try:
+    #             marketRvs = brownianMotions[self.name]
+    #         except KeyError:
+    #             raise QuantDslSyntaxError(
+    #                 "Market '%s' not available in rvs" % self.name,
+    #                 ", ".join(brownianMotions.keys()),
+    #                 node=self.node
+    #             )
+    #
+    #         try:
+    #             marketRv = marketRvs[presentTime]
+    #         except KeyError:
+    #             raise Exception, "Present time %s not in market rvs: %s" % (presentTime, marketRvs.keys())
+    #
+    #         sigma = actualHistoricalVolatility / 100
+    #         import scipy
+    #         T = getDurationYears(observationTime, presentTime)
+    #         value = lastPrice * scipy.exp(sigma * marketRv - 0.5 * sigma * sigma * T)
+    #     return value
+    #
     # def getLastPrice(self, image):
     #     return self.getMetricValue(image, 'last-price')
     #
@@ -1107,7 +1191,7 @@ class Fixing(DatedDslObject, DslExpression):
 
     def validate(self, args):
         self.assertArgsLen(args, requiredLen=2)
-        self.assertArgsPosn(args, posn=0, requiredType=(String, Date, Name, BinOp))
+        self.assertArgsPosn(args, posn=0, requiredType=(basestring, String, Date, Name, BinOp))
         self.assertArgsPosn(args, posn=1, requiredType=DslExpression)
 
     @property
@@ -1200,9 +1284,11 @@ class LongstaffSchwartz(object):
 
     def evaluate(self, **kwds):
         try:
-            brownianMotions = kwds['brownianMotions']
+            #brownianMotions = kwds['brownianMotions']
+            brownianMotions = kwds['allMarketPrices']
         except KeyError:
-            raise QuantDslSystemError("'brownianMotions' not in evaluation kwds", kwds.keys(), node=None)
+            #raise QuantDslSystemError("'brownianMotions' not in evaluation kwds", kwds.keys(), node=None)
+            raise QuantDslSystemError("'allMarketPrices' not in evaluation kwds", kwds.keys(), node=None)
         if len(brownianMotions) == 0:
             raise QuantDslSystemError('no rvs', str(kwds))
         firstMarketRvs = brownianMotions.values()[0]
@@ -1832,57 +1918,6 @@ class Underlying(DslObject):
         return self.expr
 
 
-defaultDslClasses = {
-    'Add': Add,
-    'Choice': Choice,
-    'Date': Date,
-    'Div': Div,
-    'Fixing': Fixing,
-    'Number': Number,
-    'Market': Market,
-    'Max': Max,
-    'Mul': Mult,
-    'Settlement': Settlement,
-    'String': String,
-    'Sub': Sub,
-    'UnarySub': UnarySub,
-    'Wait': Wait,
-    'On': On,
-    'Name': Name,
-    'TimeDelta': TimeDelta,
-    'Underlying': Underlying,
-    'Stub': Stub,
-}
-
-
-
-class DslNamespace(dict):
-
-    def copy(self):
-        copy = self.__class__(self)
-        return copy
-
-
-class FunctionDefCallStack(queue.Queue):
-
-    def put(self, stubId, stackedCall, stackedLocals, stackedGlobals, effectivePresentTime):
-        assert isinstance(stubId, basestring), type(stubId)
-        assert isinstance(stackedCall, FunctionDef), type(stackedCall)
-        assert isinstance(stackedLocals, DslNamespace), type(stackedLocals)
-        assert isinstance(stackedGlobals, DslNamespace), type(stackedGlobals)
-        assert isinstance(effectivePresentTime, (datetime.datetime, type(None))), type(effectivePresentTime)
-        queue.Queue.put(self, (stubId, stackedCall, stackedLocals, stackedGlobals, effectivePresentTime))
-
-
-class StubbedExpressionStack(queue.LifoQueue):
-
-    def put(self, stubId, stubbedExpr, effectivePresentTime):
-        assert isinstance(stubId, basestring), type(stubId)
-        assert isinstance(stubbedExpr, DslExpression), type(stubbedExpr)
-        assert isinstance(effectivePresentTime, (datetime.datetime, type(None))), type(effectivePresentTime)
-        queue.LifoQueue.put(self, (stubId, stubbedExpr, effectivePresentTime))
-
-
 class Module(DslObject):
     """
     A DSL module has a body, which is a list of DSL objects either
@@ -2006,6 +2041,77 @@ class Module(DslObject):
         raise QuantDslSyntaxError("shouldn't get here", node=self.node)
 
 
+defaultDslClasses = {
+    'Add': Add,
+    'And': And,
+    'Or': Or,
+    'Choice': Choice,
+    'Date': Date,
+    'Div': Div,
+    'Fixing': Fixing,
+    'Number': Number,
+    'Market': Market,
+    'Max': Max,
+    'Mult': Mult,
+    'Pow': Pow,
+    'Mod': Mod,
+    'FloorDiv': FloorDiv,
+    'Settlement': Settlement,
+    'String': String,
+    'Sub': Sub,
+    'UnarySub': UnarySub,
+    'Wait': Wait,
+    'On': On,
+    'Name': Name,
+    'TimeDelta': TimeDelta,
+    'Underlying': Underlying,
+    'Stub': Stub,
+    'Module': Module,
+    'FunctionDef': FunctionDef,
+    'FunctionCall': FunctionCall,
+    'Compare': Compare,
+    'IfExp': IfExp,
+    'If': If,
+}
+
+
+def nostub(*args):
+    """
+    Dummy 'nostub' Quant DSL decorator - we just want the name in the namespace.
+    """
+    import mock
+    return mock.Mock
+
+__all__ = defaultDslClasses.keys() + ['nostub']
+
+
+class DslNamespace(dict):
+
+    def copy(self):
+        copy = self.__class__(self)
+        return copy
+
+
+class FunctionDefCallStack(queue.Queue):
+
+    def put(self, stubId, stackedCall, stackedLocals, stackedGlobals, effectivePresentTime):
+        assert isinstance(stubId, basestring), type(stubId)
+        assert isinstance(stackedCall, FunctionDef), type(stackedCall)
+        assert isinstance(stackedLocals, DslNamespace), type(stackedLocals)
+        assert isinstance(stackedGlobals, DslNamespace), type(stackedGlobals)
+        assert isinstance(effectivePresentTime, (datetime.datetime, type(None))), type(effectivePresentTime)
+        queue.Queue.put(self, (stubId, stackedCall, stackedLocals, stackedGlobals, effectivePresentTime))
+
+
+class StubbedExpressionStack(queue.LifoQueue):
+
+    def put(self, stubId, stubbedExpr, effectivePresentTime):
+        assert isinstance(stubId, basestring), type(stubId)
+        assert isinstance(stubbedExpr, DslExpression), type(stubbedExpr)
+        assert isinstance(effectivePresentTime, (datetime.datetime, type(None))), type(effectivePresentTime)
+        queue.LifoQueue.put(self, (stubId, stubbedExpr, effectivePresentTime))
+
+
 class ExpressionStack(object):
 
     def __init__(self, rootStubId, stubbedExprs):
@@ -2279,17 +2385,42 @@ class QuantDslSystemError(QuantDslError):
     Exception class for DSL system errors.
     """
 
+class PriceProcess(object):
 
-class PriceSimulator(object):
+    __metaclass__ = ABCMeta
 
-    def getBrownianMotions(self, marketNames, fixingDates, startDate, pathCount, marketCalibration):
-        assert isinstance(startDate, datetime.datetime), type(startDate)
+    @abstractmethod
+    def simulateFuturePrices(self, marketNames, fixingDates, observationTime, pathCount, marketCalibration):
+        """
+        Returns dict (keyed by market name) of dicts (keyed by fixing date) with correlated random future prices.
+        """
+
+class BlackScholesPriceProcess(PriceProcess):
+
+    def simulateFuturePrices(self, marketNames, fixingDates, observationTime, pathCount, marketCalibration):
+        allBrownianMotions = self.getBrownianMotions(marketNames, fixingDates, observationTime, pathCount, marketCalibration)
+        # Also compute market prices, so the Market object doesn't do this.
+        allMarketPrices = {}
+        for (marketName, brownianMotions) in allBrownianMotions.items():
+            actualHistoricalVolatility = marketCalibration['%s-ACTUAL-HISTORICAL-VOLATILITY' % marketName.upper()]
+            lastPrice = marketCalibration['%s-LAST-PRICE' % marketName.upper()]
+            marketPrices = {}
+
+            for (fixingDate, brownianRv) in brownianMotions.items():
+                sigma = actualHistoricalVolatility / 100
+                T = getDurationYears(observationTime, fixingDate)
+                marketRv = lastPrice * scipy.exp(sigma * brownianRv - 0.5 * sigma * sigma * T)
+                marketPrices[fixingDate] = marketRv
+
+            allMarketPrices[marketName] = marketPrices
+        return allMarketPrices
+
+    def getBrownianMotions(self, marketNames, fixingDates, observationTime, pathCount, marketCalibration):
+        assert isinstance(observationTime, datetime.datetime), type(observationTime)
         assert isinstance(pathCount, int), type(pathCount)
 
-        import scipy
-
         marketNames = list(marketNames)
-        allDates = [startDate] + sorted(fixingDates)
+        allDates = [observationTime] + sorted(fixingDates)
 
         lenMarketNames = len(marketNames)
         lenAllDates = len(allDates)
@@ -2305,7 +2436,7 @@ class PriceSimulator(object):
                 draws = scipy.random.standard_normal(pathCount)
                 T = getDurationYears(_startDate, fixingDate)
                 if T < 0:
-                    raise QuantDslError("Can't sqrt negative numbers: %s. (Contract starts before observation time?)" % T)
+                    raise QuantDslError("Can't really square root negative time durations: %s. Contract starts before observation time?" % T)
                 endRv = startRv + scipy.sqrt(T) * draws
                 try:
                     brownianMotions[i][j + 1] = endRv
